#include "generic.h"
#include <math.h>

double sqr(double v)
{
  return v*v;
} // sqr

inline void SetPixelColor(SDL_Surface *screen, int x, int y, unsigned char red, unsigned char green, unsigned char blue)
{
  ((unsigned char*)(screen->pixels))[x*3+0 + y*screen->pitch] = red;
  ((unsigned char*)(screen->pixels))[x*3+1 + y*screen->pitch] = green;
  ((unsigned char*)(screen->pixels))[x*3+2 + y*screen->pitch] = blue;
} // SetPixelColor

inline void SetPixel(SDL_Surface *screen, int x, int y)
{
  SetPixelColor(screen, x, y, 255, 255, 255);
} // SetPixel

int main(int argc, char** argv)
{
    SDL_Surface      *screen;
    SDL_Event        event;
    int              window_width, window_height;

    window_width = 512; window_height = 512;

    if(SDL_Init(SDL_INIT_VIDEO) < 0) {
	cerr << "Could not initialize SDL video" << endl;
	exit(-1);
    } // if
    atexit(SDL_Quit);

    Uint32 flags = 0;

    // flags |= SDL_FULLSCREEN;

    screen = SDL_SetVideoMode(window_width, window_height, 24, flags);

    if(!screen) {
	cerr << "could not SetVideoMode() - abort" << endl;
	exit(-1);
    } // if

    int    x, y, a, b, radius;
    int    circle_x, circle_y;
    double dist;

    radius = 125;

    circle_x = window_width / 2; circle_y = window_height / 2;

    for(;;) {

      bzero(screen->pixels, window_width*window_height*3);

      y = radius;
      
      dist = 1.25 - radius;
      
      for(x = 0; -x/sqrt(radius*radius-x*x) > -1.1; x++) {
       	if(dist > 0) { // choose SE	
	  dist += 2*(x-y) + 5;
	  y--;
	} // if
	else {         // choose E
	  dist += 2*x+3;
	} // else

	SetPixel(screen, circle_x+x, circle_y+y);
	SetPixel(screen, circle_x-x, circle_y+y);

	SetPixel(screen, circle_x+x, circle_y-y);
	SetPixel(screen, circle_x-x, circle_y-y);


	SetPixel(screen, circle_x+y, circle_y+x);
	SetPixel(screen, circle_x-y, circle_y+x);

	SetPixel(screen, circle_x+y, circle_y-x);
	SetPixel(screen, circle_x-y, circle_y-x);
      } // for
      
      SDL_UnlockSurface(screen);
      SDL_UpdateRect(screen, 0, 0, window_width, window_height);
      SDL_LockSurface(screen);
      
      if((SDL_WaitEvent(&event) == 0) && (event.type == SDL_KEYDOWN)) 
	goto exit_app;
      
      switch(event.type) {
	
      case SDL_MOUSEMOTION:	      
	break;
	
      case SDL_MOUSEBUTTONDOWN:
	break;
	
      case SDL_MOUSEBUTTONUP:
	break;
	
      case SDL_KEYDOWN:
	
	switch(event.key.keysym.sym) {
	  
	case '+':
	  radius += 3;
	  goto cont_app;
	  break;
	  
	case '-':
	  radius -= 3;
	  goto cont_app;
	  break;
	  
	case 27:
	case 'q':
	case 'Q':
	  goto exit_app;
	  
	default:
	  break;
	} // switch
      } // switch

    cont_app:
      ;
    } // for

 exit_app:

    SDL_UnlockSurface(screen);	    
    SDL_FreeSurface(screen);

    exit(0);
} // main
